# BranchServer File templates configuration logic for SLEPOS-GUI
package POS::Admin::GUI::CR;

use strict;
use warnings;

# GUI common, includes yui
use base 'POS::Admin::GUI';
our @ISA = qw(POS::Admin::GUI);

# Data interface
use POS::Admin::GUI::Data;
use POS::Admin::GUI::FT;

our $VERSION = 0.1;

my %objects = (
    # CR
    ## global
    GLOBAL           => { LABEL => 'Basic CashRegister configuration', WIDGET => undef, VALUE => undef },
    EDIT_NAME        => { LABEL => 'CashRegister name',                WIDGET => undef, VALUE => undef },
    RADIO_GLOBAL     => { LABEL => 'Global CR',                        WIDGET => undef, VALUE => undef },
    RADIO_LOCAL      => { LABEL => 'Local CR',                         WIDGET => undef, VALUE => undef },
    RADIO_ROLE       => { LABEL => 'Role CR',                          WIDGET => undef, VALUE => undef },
    SEL_ROLE         => { LABEL => '',                                 WIDGET => undef, VALUE => undef },
    SEL_IMAGE        => { LABEL => 'Associated image',                 WIDGET => undef, VALUE => undef },
    EDIT_RAIDSCHEME  => { LABEL => 'Raid sc&heme',                     WIDGET => undef, VALUE => undef },
    CHCK_DISKJOURNAL => { LABEL => 'Enable dis&k journal',             WIDGET => undef, VALUE => undef },
    BTN_ADDCR    => { LABEL => 'Add CashRegister',           LABEL1 => 'UpdateCashRegister' },
    BTN_REMOVECR => { LABEL => 'Remove CR from list',        WIDGET => undef, VALUE => undef },
    SEL_LIST     => { LABEL => 'CashRegis&ters to register', WIDGET => undef, VALUE => undef },
    SEL_NEW      => { LABEL => '<add new CR>',               WIDGET => undef, VALUE => undef },
    ## disk drives embedded
    DISKS        => { LABEL => 'HDD/RAM disks configuration', WIDGET => undef, VALUE => undef },
    SEL_DISK     => { LABEL => 'Associated disks',            WIDGET => undef, VALUE => undef },
    SEL_DISK_NEW => { LABEL => '<add new disk>' },
    BTN_MANDISK  => { LABEL => 'Manage disks',                WIDGET => undef, VALUE => undef },
    ## disk drives full
    RADIO_HDD     => { LABEL => 'HardDisk' },
    RADIO_RAM     => { LABEL => 'RamDisk' },
    EDIT_DEVICE   => { LABEL => 'Device ID' },
    EDIT_SIZE     => { LABEL => 'Device size' },
    SEL_PART      => { LABEL => 'Partitions' },
    SEL_PART_NEW  => { LABEL => '<add new partition>' },
    EDIT_PARNUM   => { LABEL => 'Partition number' },
    EDIT_PARTYPE  => { LABEL => 'Partition type' },
    EDIT_PARMOUNT => { LABEL => 'Partition mountpoint' },
    EDIT_PARSIZE  => { LABEL => 'Partition size' },
    EDIT_PARPASS  => { LABEL => 'Partition password (encrypted partitions only)' },
    BTN_ADDPART   => { LABEL => 'Add partition', LABEL1 => 'Update partition' },
    BTN_DELPART   => { LABEL => 'Remove partition' },
    BTN_ADDDISK   => { LABEL => 'Add disk', LABEL1 => 'Update disk' },
    BTN_DELDISK   => { LABEL => 'Remove disk' },
    BTN_RETURN    => { LABEL => 'Return' }
);

my $crNameAlreadyExists    = 'Cash Register with this name already exists. Cannot rename';
my $diskNameAlreadyExists  = 'Disk drive with this name already exists. Cannot rename';
my $partNumAlreadyExists   = 'Partition with same number already exists. Cannot move';
my $partMountAlreadyExists = 'Partition with this mount point already exists. Cannot change';
my $changesExist           = 'Unsaved changes exist, selecting another image will reset them. Continue?';

sub new
{
    my $self = shift;
    my $type = ref($self) || $self;
    $self = bless {}, $type;
    $self->{'OBJECTS'}       = \%objects;
    $self->{'FACTORY'}       = shift;
    $self->{'OPTFACTORY'}    = shift;
    $self->{'ENV'}           = shift;
    $self->{'SELECTION'}     = $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'};
    $self->{'SELECTIONDISK'} = $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'};
    $self->{'SELECTIONPART'} = $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'};
    $self->{'DIRTY'}         = 0;
    # init filetemplate minimodule
    $self->{'FT'} = POS::Admin::GUI::FT->new($self->{'FACTORY'}, $self->{'OPTFACTORY'}, $self->{'ENV'});
    return $self;
} ## end sub new

sub initData
{
    my ($self, $data) = @_;
    $self->{'DATA'} = $data;
    # FIXME: when frontent is updated to support multiple BSs, update this
    my $bslist = $self->{'DATA'}->getBranchServerList();
    $self->{'BRANCHDN'} = $bslist->[0];
    my $crs = {
        $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'} => {
            GLOBAL      => 1,
            LOCAL       => 0,
            ROLE        => undef,
            IMAGE       => undef,
            DISKJOURNAL => 0,
            RAIDSCHEME  => '',
            TEMPLATES   => {},
            DISKS       => {},
            NAME        => ''
        }
    };
    my $crinfo = $self->{'DATA'}->getCashRegisters($self->{'BRANCHDN'});
    foreach my $cr (keys %$crinfo) {
        $crs->{$cr} = {
            RAIDSCHEME  => $crinfo->{$cr}{'scRaidScheme'},
            DISKJOURNAL => $crinfo->{$cr}{'scDiskJournal'} eq 'TRUE' ? 1 : 0,
        };
        $crs->{$cr}{'IMAGE'} = $crinfo->{$cr}{'scPosImageDn'};
        # note if this image does not exist (is not known to GUI), -none- is displayed in CR tab
        if ($crinfo->{$cr}{'ROLE'} !~ /global|local/) {
            $crs->{$cr}{'ROLE'} = $crinfo->{$cr}{'ROLE'};
        }
        else {
            $crs->{$cr}{'GLOBAL'} = $crinfo->{$cr}{'ROLE'} eq 'global' ? 1 : 0;
            $crs->{$cr}{'LOCAL'}  = $crinfo->{$cr}{'ROLE'} eq 'local'  ? 1 : 0;
        }
        $crs->{$cr}{'DISKS'}     = $crinfo->{$cr}{'DISKS'};
        $crs->{$cr}{'TEMPLATES'} = $crinfo->{$cr}{'TEMPLATES'};
        $crs->{$cr}{'NAME'}      = $crinfo->{$cr}{'scCashRegisterName'};
        $crs->{$cr}{'DN'}        = $crinfo->{$cr}{'DN'};
    }
    $self->{'CRS'} = $crs;
    $self->setValue(qw(SEL_LIST), [keys %$crs]);
    my @images = $self->{'DATA'}->getImageList($self->{'BRANCHDN'});
    my @roles  = $self->{'DATA'}->getRoleList($self->{'BRANCHDN'});
    $self->setValue(qw(SEL_IMAGE), ['-none-', @images]);
    $self->setValue(qw(SEL_ROLE),  ['-none-', @roles]);
    $self->{'FT'}->initData($data);
} ## end sub initData

sub drawGUI
{
    my ($self, $replacePoint) = @_;
    # undef all widget values - needed for value autosave
    foreach (keys %{ $self->{'OBJECTS'} }) {
        undef $self->{'OBJECTS'}{$_}{'WIDGET'};
    }
    # check state
    if ($self->{'ENV'}{'STATE'} eq 'CRft') {
        return $self->{'FT'}->drawGUI($replacePoint);
    }
    elsif ($self->{'ENV'}{'STATE'} eq 'CRdisk') {
        return $self->drawDiskGUI($replacePoint);
    }
    # draw
    my $factory  = $self->{'FACTORY'};
    my $frameBox = $factory->createVBox($replacePoint);
    $frameBox = $factory->createHBox($frameBox);
    my $selBoxCR = $self->createSelectionBox($frameBox, qw(SEL_LIST));
    $selBoxCR->setNotify();
    $selBoxCR->setWeight($yui::YD_HORIZ, 33);
    $factory->createHSpacing($frameBox);
    my $right = $factory->createVBox($frameBox);
    $right->setWeight($yui::YD_HORIZ, 66);
    $factory->createVSpacing($right);
    ## basic configuration
    my $basicCR = $self->createFrame($right, qw(GLOBAL));
    $basicCR = $factory->createVBox($basicCR);
    my $radios = $factory->createRadioButtonGroup($basicCR);
    $radios = $factory->createHBox($radios);
    $self->createRadioButton($radios, qw(RADIO_GLOBAL));
    $self->createRadioButton($radios, qw(RADIO_LOCAL));
    my $role = $self->createRadioButton($radios, qw(RADIO_ROLE));
    my $selRole = $self->createComboBox($radios, qw(SEL_ROLE));

    if (scalar @{ $self->getValue(qw(SEL_ROLE)) } == 1) {
        $role->setEnabled(0);
        $selRole->setEnabled(0);
    }
    $self->createInputField($basicCR, qw(EDIT_NAME))->setStretchable($yui::YD_HORIZ, 1);
    $self->createComboBox($basicCR, qw(SEL_IMAGE))->setStretchable($yui::YD_HORIZ, 1);

    if ($self->{'ENV'}{'ADVANCED'}) {
        $self->createInputField($basicCR, qw(EDIT_RAIDSCHEME))->setStretchable(0, 1);
        my $checkCrDiskJournal = $factory->createLeft($basicCR);
        $self->createCheckBox($checkCrDiskJournal, qw(CHCK_DISKJOURNAL));
    }
    ## hdd/ram drive
    my $diskCR = $self->createFrame($right, qw(DISKS));
    $diskCR = $factory->createHBox($diskCR);
    my $selBoxCrDisk = $self->createSelectionBox($diskCR, qw(SEL_DISK));
    my $tmp = $factory->createVBox($diskCR);
    $factory->createVSpacing($tmp);
    $self->createPushButton($tmp, qw(BTN_MANDISK))->setStretchable($yui::YD_HORIZ, 1);
    $factory->createVStretch($tmp);
    ## configuration templates
    $self->{'FT'}->drawEmbedded($right);
    ## rest
    $factory->createVStretch($right);
    $tmp = $factory->createHBox($right);
    $self->createPushButton($tmp, qw(BTN_ADDCR));
    $self->createPushButton($tmp, qw(BTN_REMOVECR));
    # select previousy selected CR
    $self->selectCashRegister($self->{'SELECTION'});
} ## end sub drawGUI

sub drawDiskGUI
{
    my ($self, $parent) = @_;
    my $factory = $self->{'FACTORY'};
    # update widget values
    $self->selectDisk();
    # draw widgets
    my $frameBox = $factory->createHBox($parent);
    $frameBox = $factory->createHBox($frameBox);
    my $selBox = $self->createSelectionBox($frameBox, qw(SEL_DISK));
    $selBox->setNotify();
    $selBox->setWeight($yui::YD_HORIZ, 33);
    $selBox->selectItem($selBox->findItem($self->{'SELECTIONDISK'}));
    $factory->createHSpacing($frameBox);
    my $right = $factory->createVBox($frameBox);
    $right->setWeight($yui::YD_HORIZ, 66);
    $factory->createVSpacing($right);
    ## basic configuration
    my $basic = $self->createFrame($right, qw(DISKS));
    $basic = $factory->createVBox($basic);
    my $radios = $factory->createRadioButtonGroup($basic);
    $radios = $factory->createHBox($radios);
    # basic disk info
    $self->createRadioButton($radios, qw(RADIO_HDD))->setNotify();
    $self->createRadioButton($radios, qw(RADIO_RAM))->setNotify();
    $factory->createHStretch($radios);
    $self->createInputField($basic, qw(EDIT_DEVICE))->setStretchable($yui::YD_HORIZ, 1);
    # hdd info
    if ($self->getValue(qw(RADIO_HDD))) {
        $self->createInputField($basic, qw(EDIT_SIZE))->setStretchable($yui::YD_HORIZ, 1);
        my $hddBox = $factory->createHBox($basic);
        my $selBoxPar = $self->createSelectionBox($hddBox, qw(SEL_PART));
        $selBoxPar->setNotify();
        $selBoxPar->setWeight($yui::YD_HORIZ, 33);
        $selBoxPar->selectItem($selBoxPar->findItem($self->{'SELECTIONPART'}));
        $hddBox = $factory->createVBox($hddBox);
        $self->createInputField($hddBox, qw(EDIT_PARNUM))->setStretchable($yui::YD_HORIZ, 1);
        $self->createInputField($hddBox, qw(EDIT_PARTYPE))->setStretchable($yui::YD_HORIZ, 1);
        $self->createInputField($hddBox, qw(EDIT_PARMOUNT))->setStretchable($yui::YD_HORIZ, 1);
        $self->createInputField($hddBox, qw(EDIT_PARSIZE))->setStretchable($yui::YD_HORIZ, 1);
        $self->createInputField($hddBox, qw(EDIT_PARPASS))->setStretchable($yui::YD_HORIZ, 1);
        my $tmp = $factory->createHBox($hddBox);
        $self->createPushButton($tmp, qw(BTN_ADDPART))->setStretchable($yui::YD_HORIZ, 1);

        if ($self->{'SELECTIONPART'} ne $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'}) {
            $self->getWidget(qw(BTN_ADDPART))->setLabel($self->{'OBJECTS'}{'BTN_ADDPART'}{'LABEL1'});
        }
        $self->createPushButton($tmp, qw(BTN_DELPART))->setStretchable($yui::YD_HORIZ, 1);
        $factory->createVStretch($hddBox);
    }
    ## rest
    my $tmp = $factory->createHBox($basic);
    $self->createPushButton($tmp, qw(BTN_ADDDISK))->setStretchable($yui::YD_HORIZ, 1);
    if ($self->{'SELECTIONDISK'} ne $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'}) {
        $self->getWidget(qw(BTN_ADDDISK))->setLabel($self->{'OBJECTS'}{'BTN_ADDDISK'}{'LABEL1'});
    }
    $self->createPushButton($tmp, qw(BTN_DELDISK))->setStretchable($yui::YD_HORIZ, 1);
    $self->createPushButton($basic, qw(BTN_RETURN))->setStretchable($yui::YD_HORIZ, 1);
    $factory->createVStretch($basic);
} ## end sub drawDiskGUI


sub selectCashRegister
{
    my ($self, $name) = @_;
    return if ($self->{'ENV'}{'STATE'} ne 'CR');
    $name = $self->{'SELECTION'} if (not $name);
    if ($name eq $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'}) {
        $self->getWidget(qw(BTN_ADDCR))->setLabel($self->{'OBJECTS'}{'BTN_ADDCR'}{'LABEL'});
    }
    else {
        $self->getWidget(qw(BTN_ADDCR))->setLabel($self->{'OBJECTS'}{'BTN_ADDCR'}{'LABEL1'});
    }
    return if ($self->{'DIRTY'});
    if (not $self->getWidget(qw(SEL_LIST))->findItem($name)) {
        #CR is in process being added, load <new image> data
        $name = $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'};
    }
    $self->setValue(qw(EDIT_NAME),    $self->{'CRS'}{$name}{'NAME'});
    $self->setValue(qw(RADIO_GLOBAL), $self->{'CRS'}{$name}{'GLOBAL'});
    $self->setValue(qw(RADIO_LOCAL),  $self->{'CRS'}{$name}{'LOCAL'});
    $self->setValue(qw(RADIO_ROLE),   defined $self->{'CRS'}{$name}{'ROLE'} ? 1 : 0);
    $self->setSelection(qw(SEL_ROLE), $self->{'CRS'}{$name}{'ROLE'});
    $self->setValue(qw(EDIT_RAIDSCHEME),  $self->{'CRS'}{$name}{'RAIDSCHEME'});
    $self->setValue(qw(CHCK_DISKJOURNAL), $self->{'CRS'}{$name}{'DISKJOURNAL'});
    $self->setSelection(qw(SEL_IMAGE), $self->{'CRS'}{$name}{'IMAGE'});
    $self->setValue(qw(SEL_DISK), [sort keys %{ $self->{'CRS'}{$name}{'DISKS'} }]);
    $self->updateAllWidgetValue();
    $self->{'FT'}->updateData($self->{'CRS'}{$name}{'TEMPLATES'});
    $self->setSelection(qw(SEL_LIST), $name);
} ## end sub selectCashRegister

sub selectDisk
{
    my ($self) = @_;
    return if ($self->{'ENV'}{'STATE'} ne 'CRdisk');
    my $disk = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'SELECTIONDISK'} };
    # set disk name
    $self->setValue(qw(EDIT_DEVICE), $disk->{'scDevice'});
    if ($disk->{'TYPE'} eq 'scHarddisk') {
        # if hdd -> set disk size and partition list, selectPartition()
        $self->setValue(qw(EDIT_SIZE), $disk->{'scHdSize'});
        $self->setValue(qw(RADIO_HDD), 1);
        $self->setValue(qw(RADIO_RAM), 0);
        $self->selectPartition();
    }
    else {
        $self->setValue(qw(RADIO_HDD), 0);
        $self->setValue(qw(RADIO_RAM), 1);
    }
} ## end sub selectDisk

sub selectPartition
{
    my ($self, $updateWidgets) = @_;
    return if ($self->{'ENV'}{'STATE'} ne 'CRdisk');
    my $crname    = $self->{'SELECTION'};
    my $diskname  = $self->{'SELECTIONDISK'};
    my $name      = $self->{'SELECTIONPART'};
    my $partition = $self->{'CRS'}{$crname}{'DISKS'}{$diskname}{'PARTS'}{$name};
    # always update partition selection widget
    $self->setValue(
        qw(SEL_PART),
        [   $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'},
            sort keys %{ $self->{'CRS'}{$crname}{'DISKS'}{$diskname}{'PARTS'} }
        ]
    );
    if (defined $partition) {
        $self->setValue(qw(EDIT_PARNUM),   $partition->{'scPartNum'});
        $self->setValue(qw(EDIT_PARTYPE),  $partition->{'scPartType'});
        $self->setValue(qw(EDIT_PARMOUNT), $partition->{'scPartMount'});
        $self->setValue(qw(EDIT_PARSIZE),  $partition->{'scPartSize'});
        $self->setValue(qw(EDIT_PARPASS),  $partition->{'scPassword'});
    }
    else {
        $self->setValue(qw(EDIT_PARNUM),   '');
        $self->setValue(qw(EDIT_PARTYPE),  '');
        $self->setValue(qw(EDIT_PARMOUNT), '');
        $self->setValue(qw(EDIT_PARSIZE),  '');
        $self->setValue(qw(EDIT_PARPASS),  '');
    }
    if ($updateWidgets) {
        $self->updateWidgetValue(qw(EDIT_PARNUM));
        $self->updateWidgetValue(qw(EDIT_PARTYPE));
        $self->updateWidgetValue(qw(EDIT_PARSIZE));
        $self->updateWidgetValue(qw(EDIT_PARMOUNT));
        $self->updateWidgetValue(qw(EDIT_PARPASS));
        $self->updateSelectionWidgetValue(qw(SEL_PART));
        my $selPar = $self->getWidget(qw(SEL_PART));
        $selPar->selectItem($selPar->findItem($self->{'SELECTIONPART'}));
    }
} ## end sub selectPartition

sub storeData
{
    my ($self, $data) = @_;
    # store values from widget to internal struct
    $self->storeAllWidgetValue() if ($self->{'ENV'}{'STATE'} eq 'CR');
    my $crinfo = {};
    foreach my $cr (keys %{ $self->{'CRS'} }) {
        next if ($cr eq $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'});
        $crinfo->{$cr} = {
            scRaidScheme       => $self->{'CRS'}{$cr}{'RAIDSCHEME'},
            scDiskJournal      => $self->{'CRS'}{$cr}{'DISKJOURNAL'} ? 'TRUE' : 'FALSE',
            scPosImageDn       => $self->{'CRS'}{$cr}{'IMAGE'},
            scCashRegisterName => $self->{'CRS'}{$cr}{'NAME'},
            DN                 => $self->{'CRS'}{$cr}{'DN'}
        };
        # get proper image dn for given image name
        if (defined $self->{'CRS'}{$cr}{'ROLE'}) {
            $crinfo->{$cr}{'ROLE'} = $self->{'CRS'}{$cr}{'ROLE'};
        }
        else {
            $crinfo->{$cr}{'ROLE'} = $self->{'CRS'}{$cr}{'GLOBAL'} ? 'global' : 'local';
        }
        $crinfo->{$cr}{'DISKS'}     = $self->{'CRS'}{$cr}{'DISKS'};
        $crinfo->{$cr}{'TEMPLATES'} = $self->{'CRS'}{$cr}{'TEMPLATES'};
    }
    $data->updateCashRegisters($crinfo, $self->{'BRANCHDN'});
    return 1;
} ## end sub storeData

sub updateData
{
    my ($self) = @_;
} ## end sub updateData

sub processEvent
{
    my ($self, $event) = @_;
    $self->storeAllWidgetValue();
    my $widget = $event->widget();
    if ($self->{'ENV'}{'STATE'} eq 'CR') {
        if ($widget == $self->getWidget(qw(BTN_ADDCR))) {
            # add new CR to SELLIST, select it, store data to internal datastruct
            my $selBox = $self->getWidget(qw(SEL_LIST));
            if ($self->{'DIRTY'}) {
                $self->{'DIRTY'} = 0;
            }
            else {
                # no change, we can ignore that
                return;
            }
            # get selected item in selection widget
            my $selectedItem = $self->{'SELECTION'};
            # get name filled in service name input
            my $crName = $self->getWidgetValue(qw(EDIT_NAME));
            #check name
            return if (not $crName);
            # templates tmpvar
            my $templates = $self->{'CRS'}{$selectedItem}{'TEMPLATES'};
            my $disks     = $self->{'CRS'}{$selectedItem}{'DISKS'};
            # CR id = $name;$role
            my $role;
            if ($self->getWidgetValue(qw(RADIO_LOCAL))) {
                $role = 'local';
            }
            elsif ($self->getWidgetValue(qw(RADIO_GLOBAL))) {
                $role = 'global';
            }
            else {
                $role = $self->getWidget(qw(SEL_ROLE))->selectedItem()->label();
            }
            my $crId = "${crName};${role}";
            if ($selectedItem ne $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'}) {
                if ($selectedItem ne $crId) {
                    #name changed, check uniqueness
                    if (defined $self->{'CRS'}{$crId}) {
                        $self->okPopup($crNameAlreadyExists);
                        return;
                    }
                    #preserve DN to track movement
                    $self->{'CRS'}{$crId}{'DN'} = $self->{'CRS'}{$selectedItem}{'DN'};
                    #rename, delete old name
                    delete $self->{'CRS'}{$selectedItem};
                    my $list = $self->getValue(qw(SEL_LIST));
                    for (my $i = 0; $i < scalar @$list; $i++) {
                        if ($list->[$i] eq $selectedItem) {
                            $list->[$i] = $crId;
                            last;
                        }
                    }
                    $self->setValue(qw(SEL_LIST), $list);
                }
            }
            else {
                # adding new
                #check if unique
                if (defined $self->{'CRS'}{$crId}) {
                    $self->okPopup($crNameAlreadyExists);
                    next;
                }
                my $list = $self->getValue(qw(SEL_LIST));
                push @$list, $crId;
                $self->setValue(qw(SEL_LIST), $list);
                # newly created images may have templates in <add new template> hash, get it and clear orig.
                $templates = $self->{'CRS'}{ $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'} }{'TEMPLATES'};
                $disks     = $self->{'CRS'}{ $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'} }{'DISKS'};
                $self->{'CRS'}{ $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'} }{'TEMPLATES'} = {};
                $self->{'CRS'}{ $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'} }{'DISKS'}     = {};
            }
            # add new data (when either adding new or renaming)
            my $image = $self->getWidget(qw(SEL_IMAGE))->selectedItem();
            if (defined $image) {
                $image = $image->label();
            }
            $self->{'CRS'}{$crId} = {
                RAIDSCHEME => $self->{'ENV'}{'ADVANCED'} ? $self->getWidgetValue(qw(EDIT_RAIDSCHEME)) :
                  $self->getValue(qw(EDIT_RAIDSCHEME)),
                DISKJOURNAL => $self->{'ENV'}{'ADVANCED'} ? $self->getWidgetValue(qw(CHCK_DISKJOURNAL)) :
                  $self->getValue(qw(CHCK_DISKJOURNAL)),
                LOCAL  => $role eq 'local'  ? 1 : 0,
                GLOBAL => $role eq 'global' ? 1 : 0,
                ROLE => $role !~ /global|local/ ? $role : undef,
                IMAGE     => $image,
                TEMPLATES => $templates,
                DISKS     => $disks,
                NAME      => $crName
            };
            # update selList
            $self->updateSelectionWidgetValue(qw(SEL_LIST));
            $self->{'SELECTION'} = $crId;
            $self->selectCashRegister();
        }
        ## Remove CR
        elsif ($widget == $self->getWidget(qw(BTN_REMOVECR))) {
            my $selBox = $self->getWidget(qw(SEL_LIST));
            # get selected item in selection widget
            my $selectedItem = $self->{'SELECTION'};
            if ($selectedItem ne $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'}) {
                delete $self->{'CRS'}{$selectedItem};
                my $list    = $self->getValue(qw(SEL_LIST));
                my $newlist = [];
                for (my $i = 0; $i < scalar @$list; $i++) {
                    if ($list->[$i] ne $selectedItem) {
                        push @$newlist, $list->[$i];
                    }
                }
                $self->setValue(qw(SEL_LIST), $newlist);
                $self->{'SELECTION'} = $self->{'OBJECTS'}{'SEL_NEW'}{'LABEL'};
                $self->selectCashRegister();
            }
        }
        ## Select CR
        elsif ($widget == $self->getWidget(qw(SEL_LIST))) {
            if ($self->{'DIRTY'}) {
                if ($self->yesnoPopup($changesExist)) {
                    $self->{'DIRTY'} = 0;
                }
                else {
                    #reset selection
                    my $selBox = $self->getWidget(qw(SEL_LIST));
                    if (defined $selBox) {
                        $selBox->selectItem($selBox->findItem($self->{'SELECTION'}));
                    }
                    return;
                }
            }
            $self->{'SELECTION'} = $self->getWidget(qw(SEL_LIST))->selectedItem()->label();
            $self->selectCashRegister();
        }
        ## manage disk button
        elsif ($widget == $self->getWidget(qw(BTN_MANDISK))) {
            # switch state to CRdisk
            $self->{'ENV'}{'STATE'} = 'CRdisk';
            #add <add new disk> to DISKS
            $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'} } =
              { TYPE => 'scHarddisk' };
            $self->setValue(qw(SEL_DISK), [sort keys %{ $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'} }]);
            return;
        }
        else {
            $self->{'DIRTY'} = 1;
            my $selRole  = $self->getWidget(qw(SEL_ROLE));
            my $selImage = $self->getWidget(qw(SEL_IMAGE));
            if ($widget == $selImage) {
                $self->setSelection(qw(SEL_IMAGE), $selImage->selectedItem()->label());
            }
            elsif ($widget == $selRole) {
                $self->setSelection(qw(SEL_ROLE), $selRole->selectedItem()->label());
            }
            else {
                return $self->{'FT'}->processEvent($event);
            }
            return;
        }
    }
    ## process disks adition/removal
    elsif ($self->{'ENV'}{'STATE'} eq 'CRdisk') {
        if ($widget == $self->getWidget(qw(RADIO_HDD))) {
            # save data so they are not lost during hdd<-ramdisk switch
            my $disk = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'SELECTIONDISK'} };
            $disk->{'scDevice'} = $self->getWidgetValue(qw(EDIT_DEVICE));
            if (defined $self->getWidget(qw(EDIT_SIZE))) {
                # we are actually not switching anywhere (hdd->hdd)
                $disk->{'scHdSize'} = $self->getWidgetValue(qw(EDIT_SIZE));
            }
            $disk->{'TYPE'} = 'scHarddisk';
            return 1;
        }
        elsif ($widget == $self->getWidget(qw(RADIO_RAM))) {
            # save data so they are not lost during hdd->ramdisk switch
            my $disk = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'SELECTIONDISK'} };
            $disk->{'scDevice'} = $self->getWidgetValue(qw(EDIT_DEVICE));
            if (defined $self->getWidget(qw(EDIT_SIZE))) {
                # now we are switching hdd->ramdisk
                $disk->{'scHdSize'} = $self->getWidgetValue(qw(EDIT_SIZE));
            }
            $disk->{'TYPE'} = 'scRamDisk';
            return 1;
        }
        elsif ($widget == $self->getWidget(qw(BTN_RETURN))) {
            #remove <add new disk> from DISKS, revert SELECTIONDISK and PART so it is not interfering with other CRs
            $self->{'SELECTIONDISK'} = $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'};
            $self->{'SELECTIONPART'} = $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'};
            delete $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'SELECTIONDISK'} };
            $self->setValue(qw(SEL_DISK), [sort keys %{ $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'} }]);
            $self->{'ENV'}{'STATE'} = 'CR';
            #return undef, we change states
            return;
        }
        elsif ($widget == $self->getWidget(qw(SEL_DISK))) {
            if ($self->{'SELECTIONDISK'} ne $self->getWidget(qw(SEL_DISK))->selectedItem()->label()) {
                $self->{'SELECTIONDISK'} = $self->getWidget(qw(SEL_DISK))->selectedItem()->label();
                #revert SELECTIONPART to <add new part>, so it is not interfering with other disks
                $self->{'SELECTIONPART'} = $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'};
                #return 1 to trigger redraw
                return 1;
            }
            return;
        }
        elsif ($widget == $self->getWidget(qw(SEL_PART))) {
            if ($self->{'SELECTIONPART'} ne $self->getWidget(qw(SEL_PART))->selectedItem()->label()) {
                $self->{'SELECTIONPART'} = $self->getWidget(qw(SEL_PART))->selectedItem()->label();
                $self->selectPartition(1);
            }
            #return undef, we dont need redraw
            return;
        }
        elsif ($widget == $self->getWidget(qw(BTN_ADDDISK))) {
            # get name filled in service name input
            my $diskName = $self->getWidgetValue(qw(EDIT_DEVICE));
            # disks shortcut
            my $disks = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'};
            # partitions tmp
            my $partitions = $disks->{ $self->{'SELECTIONDISK'} }{'PARTS'};
            if ($self->{'SELECTIONDISK'} ne $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'}) {
                if ($self->{'SELECTIONDISK'} ne $diskName) {
                    #name changed, check uniqueness
                    if (defined $disks->{$diskName}) {
                        $self->okPopup($diskNameAlreadyExists);
                        return;
                    }
                    #rename, delete old name
                    delete $disks->{ $self->{'SELECTIONDISK'} };
                    my $list = $self->getValue(qw(SEL_DISK));
                    for (my $i = 0; $i < scalar @$list; $i++) {
                        if ($list->[$i] eq $self->{'SELECTIONDISK'}) {
                            $list->[$i] = $diskName;
                            last;
                        }
                    }
                    $self->setValue(qw(SEL_DISK), [sort @$list]);
                }
            }
            else {
                # adding new
                #check if unique
                if (defined $disks->{$diskName}) {
                    $self->okPopup($diskNameAlreadyExists);
                    return;
                }
                my $list = $self->getValue(qw(SEL_DISK));
                push @$list, $diskName;
                $self->setValue(qw(SEL_DISK), [sort @$list]);
                # newly created disks may have partitions in <add new disk> hash, get it and clear orig.
                # SELECTIONDISK now points to <add new disk>
                $disks->{ $self->{'SELECTIONDISK'} } =
                  { PARTS => {}, TYPE => 'scHarddisk', scDevice => '', scHdSize => '' };
            }
            # store data into internal struct
            $disks->{$diskName} = {
                PARTS    => $partitions,
                TYPE     => $self->getWidgetValue(qw(RADIO_HDD)) ? 'scHarddisk' : 'scRamDisk',
                scDevice => $self->getWidgetValue(qw(EDIT_DEVICE)),
                scHdSize => $self->getWidgetValue(qw(RADIO_HDD)) ? $self->getWidgetValue(qw(EDIT_SIZE)) : ''
            };
            $self->{'SELECTIONDISK'} = $diskName;
            return 1;
        }
        elsif ($widget == $self->getWidget(qw(BTN_DELDISK))) {
            my $selectedItem = $self->{'SELECTIONDISK'};
            if ($selectedItem ne $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'}) {
                my $disks = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'};
                delete $disks->{$selectedItem};
                $self->{'SELECTIONDISK'} = $self->{'OBJECTS'}{'SEL_DISK_NEW'}{'LABEL'};
                $self->{'SELECTIONPART'} = $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'};
                my @list = grep(!/$selectedItem/, @{ $self->getValue(qw(SEL_DISK)) });
                $self->setValue(qw(SEL_DISK), [sort @list]);
                return 1;
            }
            return;
        }
        elsif ($widget == $self->getWidget(qw(BTN_ADDPART))) {
            # get name filled in service name input
            my $partName = $self->getWidgetValue(qw(EDIT_PARNUM)) . " - " . $self->getWidgetValue(qw(EDIT_PARMOUNT));
            # disks shortcut
            my $parts = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'SELECTIONDISK'} }{'PARTS'};
            if ($self->{'SELECTIONPART'} ne $partName) {
                #name changed or adding new
                foreach (keys %$parts) {
                    my ($pn, $pm) = split(/ - /, $_);
                    #check partition number uniqueness
                    if ($pn eq $self->getWidgetValue(qw(EDIT_PARNUM))) {
                        $self->okPopup($partNumAlreadyExists);
                        return;
                    }
                    #check partition mount point uniqueness
                    if ($pm eq $self->getWidgetValue(qw(EDIT_PARMOUNT))) {
                        $self->okPopup($partMountAlreadyExists);
                        return;
                    }
                }
                #when renaming, delete old name, new will be added later
                #when adding this will delete <add new part> -> will add later
                delete $parts->{ $self->{'SELECTIONPART'} };
            }
            # store data into internal struct
            $parts->{$partName} = {
                scPartNum   => $self->getWidgetValue(qw(EDIT_PARNUM)),
                scPartType  => $self->getWidgetValue(qw(EDIT_PARTYPE)),
                scPartMount => $self->getWidgetValue(qw(EDIT_PARMOUNT)),
                scPartSize  => $self->getWidgetValue(qw(EDIT_PARSIZE)),
                scPassword  => $self->getWidgetValue(qw(EDIT_PARPASS))
            };
            $self->{'SELECTIONPART'} = $partName;
            $self->selectPartition(1);
            return;
        }
        elsif ($widget == $self->getWidget(qw(BTN_DELPART))) {
            my $selectedItem = $self->{'SELECTIONPART'};
            if ($selectedItem ne $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'}) {
                my $parts = $self->{'CRS'}{ $self->{'SELECTION'} }{'DISKS'}{ $self->{'SELECTIONDISK'} }{'PARTS'};
                delete $parts->{$selectedItem};
                $self->{'SELECTIONPART'} = $self->{'OBJECTS'}{'SEL_PART_NEW'}{'LABEL'};
                $self->selectPartition(1);
                return;
            }
            return;
        }
    }
    ## process possible file templates events
    elsif ($self->{'ENV'}{'STATE'} eq 'CRft') {
        return $self->{'FT'}->processEvent($event);
    }
} ## end sub processEvent

1;
