# BranchServer File templates configuration logic for SLEPOS-GUI
package POS::Admin::GUI::FT;

use strict;
use warnings;

# GUI common, includes yui
use base 'POS::Admin::GUI';
our @ISA = qw(POS::Admin::GUI);
# Data interface
use POS::Admin::GUI::Data;

our $VERSION = 0.1;

my %objects = (
    # IM
    # config templates
    EDIT_NAME       => { LABEL => 'Template name',         WIDGET => undef, VALUE => undef },
    EDIT_TARGETFILE => { LABEL => 'Target file',           WIDGET => undef, VALUE => undef },
    EDIT_FILENAME   => { LABEL => 'Source fi&le',          WIDGET => undef, VALUE => undef },
    RADIO_FILE      => { LABEL => 'Static file template',  WIDGET => undef, VALUE => undef },
    RADIO_FILESYNC  => { LABEL => 'Dynamic file template', WIDGET => undef, VALUE => undef },
    RADIO_PXE       => { LABEL => 'PXE file template',     WIDGET => undef, VALUE => undef },
    SEL_LIST        => { LABEL => 'Registered templates',  WIDGET => undef, VALUE => undef },
    CHCK_MUST       => { LABEL => 'Template enabled',      WIDGET => undef, VALUE => undef },
    EDIT_BSIZE =>
      { LABEL => 'Bloc&ksize depended of the file size, 1024, 2048,4096, 8192', WIDGET => undef, VALUE => undef },
    BTN_MANAGETEMPL => { LABEL => 'Manage file templates',       WIDGET => undef, VALUE => undef },
    BTN_RETURN      => { LABEL => 'Return',                      WIDGET => undef, VALUE => undef },
    BTN_ADDTEMPLATE => { LABEL => 'Add template',                LABEL1 => 'Update template' },
    BTN_DELTEMPLATE => { LABEL => 'Remove template',             WIDGET => undef, VALUE => undef },
    SEL_NEW         => { LABEL => '<add new template>',          WIDGET => undef, VALUE => undef },
    GLOBAL          => { LABEL => 'File template configuration', WIDGET => undef, VALUE => undef },
    BTN_FILENAME    => { LABEL => 'S&elect File',                WIDGET => undef, VALUE => undef },
    TEMPLATES       => { LABEL => 'File templates',              WIDGET => undef, VALUE => undef }
);

my $templateNameAlreadyExists = 'Template with this name already exists. Cannot rename';

sub new
{
    my $self = shift;
    my $type = ref($self) || $self;
    $self = bless {}, $type;
    $self->{'OBJECTS'}    = \%objects;
    $self->{'FACTORY'}    = shift;
    $self->{'OPTFACTORY'} = shift;
    $self->{'ENV'}        = shift;
    return $self;
} ## end sub new

sub initData
{
    my ($self, $data) = @_;
} ## end sub initData

## used both for embedded gui data update.
sub updateData
{
    my ($self, $data) = @_;
    if (defined $data) {
       # update DATA member variable which points to cr/im->TEMPLATES so operations results are stored directly to im/cr
        $self->{'DATA'} = $data;
        # update SEL_LIST value so drawEmbedded displays valid data
        $self->setValue(qw(SEL_LIST), [sort keys %$data]);
    }
    else {
        $self->{'DATA'} = {};
        $self->setValue(qw(SEL_LIST), []);
    }
    # visualise change
    $self->updateAllWidgetValue();
} ## end sub updateData

sub drawGUI
{
    my ($self, $replacePoint) = @_;
    # undef all widget values - needed for value autosave
    foreach (keys %{ $self->{'OBJECTS'} }) {
        undef $self->{'OBJECTS'}{$_}{'WIDGET'};
    }
    my $factory = $self->{'FACTORY'};
    my $tmp     = $factory->createHBox($replacePoint);
    # file templates selection box
    my $sellist = $self->createSelectionBox($tmp, qw(SEL_LIST));
    $sellist->setNotify();
    $sellist->setWeight($yui::YD_HORIZ, 33);
    $factory->createHSpacing($tmp);
    my $right = $factory->createVBox($tmp);
    $right->setWeight($yui::YD_HORIZ, 66);
    $factory->createVSpacing($right);
    # radio buttons
    my $basicFT = $self->createFrame($right, qw(GLOBAL));
    $basicFT = $factory->createVBox($basicFT);
    my $radios = $factory->createRadioButtonGroup($basicFT);
    $radios = $factory->createHBox($radios);
    $self->createRadioButton($radios, qw(RADIO_FILE));
    $factory->createHSpacing($radios);
    $self->createRadioButton($radios, qw(RADIO_FILESYNC));
    $factory->createHSpacing($radios);
    $self->createRadioButton($radios, qw(RADIO_PXE));
    # edit boxes
    $self->createInputField($basicFT, qw(EDIT_NAME))->setStretchable($yui::YD_HORIZ, 1);
    $self->createInputField($basicFT, qw(EDIT_FILENAME))->setStretchable($yui::YD_HORIZ, 1);
    $self->createInputField($basicFT, qw(EDIT_TARGETFILE))->setStretchable($yui::YD_HORIZ, 1);
    $self->createInputField($basicFT, qw(EDIT_BSIZE))->setStretchable($yui::YD_HORIZ, 1);
    # check box
    $self->createCheckBox($basicFT, qw(CHCK_MUST));
    #todo add checking logic
    $tmp = $factory->createHBox($right);
    $self->createPushButton($tmp, qw(BTN_ADDTEMPLATE))->setStretchable($yui::YD_HORIZ, 1);
    $self->createPushButton($tmp, qw(BTN_DELTEMPLATE))->setStretchable($yui::YD_HORIZ, 1);
    $self->createPushButton($right, qw(BTN_RETURN))->setStretchable($yui::YD_HORIZ, 1);
    $factory->createVStretch($right);
    $self->selectTemplate();
} ## end sub drawGUI

sub drawEmbedded
{
    my ($self, $parent) = @_;
    # undef all widget values - needed for value autosave
    foreach (keys %{ $self->{'OBJECTS'} }) {
        undef $self->{'OBJECTS'}{$_}{'WIDGET'};
    }
    my $factory = $self->{'FACTORY'};
    my $conffileCR = $self->createFrame($parent, qw(TEMPLATES));
    $conffileCR = $factory->createHBox($conffileCR);
    $self->createSelectionBox($conffileCR, qw(SEL_LIST));
    my $right = $factory->createVBox($conffileCR);
    $factory->createVSpacing($right);
    $self->createPushButton($right, qw(BTN_MANAGETEMPL))->setStretchable($yui::YD_HORIZ, 1);
    $factory->createVStretch($right);
} ## end sub drawEmbedded

sub selectTemplate
{
    my ($self, $name) = @_;
    if ((not defined $name) or ($name eq $self->{'OBJECTS'}{SEL_NEW}{'LABEL'})) {
        $name = $self->{'OBJECTS'}{SEL_NEW}{'LABEL'};
        $self->setValue(qw(EDIT_NAME), '');
        $self->getWidget(qw(BTN_ADDTEMPLATE))->setLabel($self->{'OBJECTS'}{BTN_ADDTEMPLATE}{'LABEL'});
    }
    else {
        $self->setValue(qw(EDIT_NAME), $name);
        $self->getWidget(qw(BTN_ADDTEMPLATE))->setLabel($self->{'OBJECTS'}{BTN_ADDTEMPLATE}{'LABEL1'});
    }
    if ($self->{'DATA'}{$name}{'ROLE'} eq 'FT') {
        $self->setValue(qw(RADIO_FILE),    1);
        $self->setValue(qw(EDIT_FILENAME), $self->{'DATA'}{$name}{'scConfigFileData'});
    }
    elsif ($self->{'DATA'}{$name}{'ROLE'} eq 'FST') {
        $self->setValue(qw(RADIO_FILESYNC), 1);
        $self->setValue(qw(EDIT_FILENAME),  $self->{'DATA'}{$name}{'scConfigFileLocalPath'});
    }
    elsif ($self->{'DATA'}{$name}{'ROLE'} eq 'PT') {
        $self->setValue(qw(RADIO_PXE), 1);
        #todo kernelParameters
    }
    $self->setValue(qw(CHCK_MUST), $self->{'DATA'}{$name}{'scMust'} eq 'TRUE' ? 1 : 0);
    $self->setValue(qw(EDIT_TARGETFILE), $self->{'DATA'}{$name}{'scConfigFile'});
    $self->setValue(qw(EDIT_BSIZE),      $self->{'DATA'}{$name}{'scBsize'});
    $self->updateAllWidgetValue();
    my $selBox = $self->getWidget(qw(SEL_LIST));
    if (defined $selBox) {
        $selBox->selectItem($selBox->findItem($name));
    }
} ## end sub selectTemplate

sub processEvent
{
    my ($self, $event) = @_;
    #get basic state
    my $widget = $event->widget();
    my ($bstate) = $self->{'ENV'}{STATE} =~ /([^ft]*)f?t?/;
    if ($bstate eq $self->{'ENV'}{STATE}) {
        #just check if widget is MANAGETEMPL
        if ($widget == $self->getWidget(qw(BTN_MANAGETEMPL))) {
            #add <add new template> to template list, so it is displayed in full form
            if (not defined $self->{'DATA'}{ $self->{'OBJECTS'}{SEL_NEW}{'LABEL'} }) {
                # add <add new template> default data
                $self->{'DATA'}{ $self->{'OBJECTS'}{SEL_NEW}{'LABEL'} } =
                  { ROLE => 'FT', scMust => 'TRUE', scConfigFile => '', scBsize => '8084', scConfigFileData => '' };
            }
            # update SEL_LIST value so drawGUI displays valid data
            $self->setValue(qw(SEL_LIST), [sort keys %{ $self->{'DATA'} }]);
            $self->{'ENV'}{STATE} = $bstate . 'ft';
            return;
        }
    }
    else {
        # check return
        if ($widget == $self->getWidget(qw(BTN_RETURN))) {
            #remove <add new template> from list, so it is not displayed in embedded form
            if (defined $self->{'DATA'}{ $self->{'OBJECTS'}{SEL_NEW}{'LABEL'} }) {
                delete $self->{'DATA'}{ $self->{'OBJECTS'}{SEL_NEW}{'LABEL'} };
                # update SEL_LIST value so drawEmbedded displays valid data
                $self->setValue(qw(SEL_LIST), [sort keys %{ $self->{'DATA'} }]);
            }
            $self->{'ENV'}{STATE} = $bstate;
            return;
        }
        elsif ($widget == $self->getWidget(qw(SEL_LIST))) {
            $self->selectTemplate($self->getWidget(qw(SEL_LIST))->selectedItem()->label());
        }
        if ($widget == $self->getWidget(qw(BTN_ADDTEMPLATE))) {
            ## add template to SELLIST, select it, store data to internal datastruct
            my $selBox = $self->getWidget(qw(SEL_LIST));
            # get selected item in selection widget
            my $selectedItem = $selBox->selectedItem();
            # get name filled in service name input
            my $name = $self->getWidgetValue(qw(EDIT_NAME));
            #check name
            return if (not $name);
            if (defined $selectedItem && $selectedItem->label() ne $self->{'OBJECTS'}{SEL_NEW}{'LABEL'}) {
                if ($selectedItem->label() ne $name) {
                    #name changed, check uniqueness
                    if (defined $self->{'DATA'}{$name}) {
                        $self->okPopup($templateNameAlreadyExists);
                        next;
                    }
                    #rename, delete old name
                    delete $self->{'DATA'}{ $selectedItem->label() };
                    my $list = $self->getValue(qw(SEL_LIST));
                    for (my $i = 0; $i < scalar @$list; $i++) {
                        if ($list->[$i] eq $selectedItem->label()) {
                            $list->[$i] = $name;
                            last;
                        }
                    }
                    $self->setValue(qw(SEL_LIST), $list);
                }
            }
            else {
                # adding new
                #check if unique
                if (defined $self->{'DATA'}{$name}) {
                    $self->okPopup($templateNameAlreadyExists);
                    next;
                }
                my $list = $self->getValue(qw(SEL_LIST));
                push @$list, $name;
                $self->setValue(qw(SEL_LIST), $list);
            }
            # add new data (when either adding new or renaming)
            my $role;
            my $data = {};
            if ($self->getWidgetValue(qw(RADIO_FILE))) {
                $role                       = 'FT';
                $data->{'scConfigFileData'} = $self->getWidgetValue(qw(EDIT_FILENAME));
                $data->{'scConfigFile'}     = $self->getWidgetValue(qw(EDIT_TARGETFILE));
                $data->{'scBsize'}          = $self->getWidgetValue(qw(EDIT_BSIZE));
            }
            elsif ($self->getWidgetValue(qw(RADIO_FILESYNC))) {
                $role                            = 'FST';
                $data->{'scConfigFileLocalPath'} = $self->getWidgetValue(qw(EDIT_FILENAME));
                $data->{'scConfigFile'}          = $self->getWidgetValue(qw(EDIT_TARGETFILE));
                $data->{'scBsize'}               = $self->getWidgetValue(qw(EDIT_BSIZE));
            }
            elsif ($self->getWidgetValue(qw(RADIO_PXE))) {
                $role = 'PT';
                #todo kernel params
            }
            $data->{'scMust'}      = $self->getWidgetValue(qw(CHCK_MUST)) ? 'TRUE' : 'FALSE';
            $data->{'ROLE'}        = $role;
            $self->{'DATA'}{$name} = $data;
            # update selList
            $self->selectTemplate($name);
        }
        ## remove template
        elsif ($widget == $self->getWidget(qw(BTN_DELTEMPLATE))) {
            my $selBox = $self->getWidget(qw(SEL_LIST));
            # get selected item in selection widget
            my $selectedItem = $selBox->selectedItem();
            if (defined $selectedItem && $selectedItem->label() ne $self->{'OBJECTS'}{SEL_NEW}{'LABEL'}) {
                delete $self->{'DATA'}{ $selectedItem->label() };
                my $list    = $self->getValue(qw(SEL_LIST));
                my $newlist = [];
                for (my $i = 0; $i < scalar @$list; $i++) {
                    if ($list->[$i] ne $selectedItem->label()) {
                        push @$newlist, $list->[$i];
                    }
                }
                $self->setValue(qw(SEL_LIST), $newlist);
                $self->selectTemplate();
            }
        }
    }
} ## end sub processEvent

1;
