#!/usr/bin/perl -w


# uses Net::IPv4Addr to provide all IP (v4) addressing related functionality for SLEPOS
# TODO: put all code so far provided by Net::IP module here, either by Net::IPv4Addr or
# or own means
# NetAddr::IP::Liteusage, if it makes sense, should be kept


package POS::IP;

use strict;
use Net::IPv4Addr qw(:all);

our $VERSION = '1.0';


require Exporter;
our @ISA = qw(Exporter);

#our @EXPORT = qw(IP);

our @EXPORT_OK = qw(is_in_range get_ip);


## Constructor
sub new
{
    my ($package, $ip) = @_;
    my $data = { 'IP' => $ip, 'INTIP' => undef };
    my $pip = _parse($ip);    #ipv4_chkip tries to correct ipaddress if good substring found, need make sure it is same
    if (!defined $pip || $pip ne $ip) {
        return undef;
    }
    return bless($data);
} ## end sub new


## testing ip-address
sub _parse
{
    my $ip_addr = shift;
    return ipv4_chkip($ip_addr);    #should be ipv4_checkip according to CPAN, is it a bug in Net::IPv4Addr?
} ## end sub _parse


sub get_ip
{
    my ($self) = @_;
    return $self->{'IP'};
} ## end sub get_ip


sub intip
{
    my ($self) = @_;
    my $ip_num = $self->{'INTIP'};
    return $ip_num if defined $ip_num;
    $self->{'INTIP'} = $self->_intip();
    return $self->{'INTIP'};
} ## end sub intip

sub _intip    #can input different ip if needed, only for internal purposes
{
    my ($self, $ip) = @_;
    $ip = $self->get_ip() unless defined $ip;
    my $ip_num = unpack('N', pack('C4', split(/\./, $ip)));
    return $ip_num;

} ## end sub _intip


sub is_in_range
{
    my ($self, $from, $to) = @_;
    my $iip   = $self->intip();
    my $ifrom = $self->_intip($from);
    my $ito   = $self->_intip($to);

    return 1 if ($ifrom <= $iip && $iip <= $ito);
    return 0;
} ## end sub is_in_range


1;
