<?xml version="1.0" encoding='utf-8'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html" encoding="utf-8" />

<!--<xsl:output method="html" indent="yes" encoding="utf-8" />-->

<!--
  SLEPOS LDAP schema XML2XHTML transformation template
-->

<xsl:template match="/">
  <html>
  <head>
    <!-- CSS defs -->
    <style type="text/css">
    <!-- topic -->
      #title {
        background-color:#227808;border-style:none;
        color: white;
        padding: 10px;
      }
    <!-- elements -->
      #optional {
        background-color:#e0ffff;border-style:dotted;border-width:2px;margin:3px;
      }
      #mandatory {
        background-color:#b0c4de;border-style:solid;border-width:2px;margin:3px;
      }
      #oneormore {
        background-color:#6495ed;border-style:dashed;border-width:2px;margin:3px;
      }
      #choice {
        background-color:#E35001;border-style:double;border-width:2px;margin:3px;
      }
      #subelements {
        margin-left:60px;margin-right:10px;
      }
      #attributes {
        border-style:solid;border-width:1px;margin:3px;
      }
      #legend {
        border-style:solid;border-width:1px;width: 15%;float: left;
      }
      h4 {
        margin:3px;
      }
    </style>
    <!-- JS defs -->
    
    <title>SLEPOS LDAP Schema visualisation</title>
  </head>
  <body>
    <h1 id="title">SLEPOS LDAP Schema visualisation</h1>
    <div id="legend">
    <h3>Legend</h3>
      <div id="mandatory">Mandatory entry<br />(min 1, max 1)</div>
      <div id="oneormore">Mandatory entry<br />(min 1, max undefined)</div>
      <div id="optional">Optional entry<br />(min 0, max undefined)</div>
      <div id="choice">Choice entry<br />(only one entry)</div>
    </div>
    <xsl:apply-templates select="/grammar/start" />
    <hr />
    <h2>objectClass details</h2>
    <xsl:for-each select="grammar/define">
      <xsl:sort select="@name" />
      <div id="mandatory">
        <xsl:variable name="this.name"><xsl:value-of select="@name" /></xsl:variable>
        <a name="{$this.name}" /><h3><xsl:value-of select="$this.name" /></h3>
        <div id="attributes">
          <h4>Attributes:</h4>
          Mandatory:
          <xsl:for-each select="element/element[@name='attributes']/interleave/element">
            <b><xsl:value-of select="@name" />, </b>
          </xsl:for-each><br />
          Optional:
          <xsl:for-each select="element/element[@name='attributes']/interleave/zeroOrMore/element">
            <i><xsl:value-of select="@name" />, </i>
          </xsl:for-each><br />
        </div>
      </div>
    </xsl:for-each>
  </body>
  </html>
</xsl:template>

<!-- Always start with reference in start element -->
<xsl:template match="start">
    <xsl:variable name="next" select="element[@name='slepos']/ref/@name" />
    <div style="position:relative;margin-left:auto;margin-right:auto;width:70%;">
    <div id="mandatory">
      <xsl:apply-templates select="/grammar/define[@name=$next]" />
    </div>
    </div>
</xsl:template>

<!-- generic define expansion -->
<xsl:template match="define">
  <xsl:param name="parsed" />
  <xsl:variable name="this.name" select="@name"/>
  <xsl:choose>
    <xsl:when test="not(contains($parsed, $this.name))">
      <a href="#{$this.name}"><h4><xsl:value-of select="$this.name" /></h4></a>
      <xsl:if test="element//ref">
        <div id="subelements">
          <!-- Nejdriv mandatory parametry s maxOccurs eq 1 -->
          <xsl:for-each select="element/ref | element/interleave | element/oneOrMore | element/optional | element/zeroOrMore | element/choice">
            <xsl:apply-templates select=".">
              <xsl:with-param name="parsed" select="concat($parsed,$this.name)" />
            </xsl:apply-templates>
          </xsl:for-each>
          </div>
      </xsl:if>
    </xsl:when>
    <xsl:otherwise>
      <a href="#{$this.name}"><h4><xsl:value-of select="$this.name" /></h4></a>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="optional|zeroOrMore">
  <xsl:param name="parsed" />
  <div id="optional">
    <xsl:apply-templates>
      <xsl:with-param name="parsed" select="$parsed" />
    </xsl:apply-templates>
  </div>
</xsl:template>

<xsl:template match="group">
  <xsl:param name="parsed" />
  <div id="mandatory">
    <xsl:apply-templates>
      <xsl:with-param name="parsed" select="$parsed" />
    </xsl:apply-templates>
  </div>
</xsl:template>

<xsl:template match="interleave">
  <xsl:param name="parsed" />
  <xsl:apply-templates>
    <xsl:with-param name="parsed" select="$parsed" />
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="oneOrMore">
  <xsl:param name="parsed" />
  <div id="oneormore">
    <xsl:apply-templates>
      <xsl:with-param name="parsed" select="$parsed" />
    </xsl:apply-templates>
  </div>
</xsl:template>

<xsl:template match="choice">
  <xsl:param name="parsed" />
  <div id="choice">
    <xsl:apply-templates>
      <xsl:with-param name="parsed" select="$parsed" />
    </xsl:apply-templates>
  </div>
</xsl:template>

<xsl:template match="ref">
  <xsl:param name="parsed" />
  <xsl:variable name="el" select="@name" />
  <xsl:choose>
    <xsl:when test="../../element/ref[@name=$el] | ../../../element/interleave/ref[@name=$el] | ../../group/ref[@name=$el]">
      <div id="mandatory">
        <xsl:apply-templates select="/grammar/define[@name=$el]">
          <xsl:with-param name="parsed" select="$parsed" />
        </xsl:apply-templates>
      </div>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="/grammar/define[@name=$el]">
        <xsl:with-param name="parsed" select="$parsed" />
      </xsl:apply-templates>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>