<?xml version="1.0" encoding='utf-8'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml" encoding="utf-8" indent="yes"/>

<!-- SLEPOS-Kiwi old format transformation -->

<!-- copy unmatched elements -->
<xsl:template match="@*|node()">
    <xsl:copy>
        <xsl:apply-templates select="@*|node()" />
    </xsl:copy>
</xsl:template>

<!-- transform boot image paths -->
<xsl:template match="type">
    <xsl:choose>
        <xsl:when test = "@boot = '/usr/share/kiwi/image/SLEPOS/usbboot-3'">
            <xsl:copy>
                <xsl:apply-templates select="@*" />
                <xsl:text>oem</xsl:text>
            </xsl:copy>
        </xsl:when>
        <xsl:otherwise>
            <xsl:copy>
                <xsl:apply-templates select="@*|node()" />
            </xsl:copy>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template match="type/@boot">
    <xsl:attribute name="boot">
        <xsl:choose>
            <xsl:when test="current() = '/usr/share/kiwi/image/SLEPOS/netboot-3'">
                <xsl:text>netboot/suse-SLES11</xsl:text>
            </xsl:when>
            <xsl:when test="current() = '/usr/share/kiwi/image/SLEPOS/isoboot-3'">
                <xsl:text>isoboot/suse-SLES11</xsl:text>
            </xsl:when>
            <xsl:when test="current() = '/usr/share/kiwi/image/SLEPOS/usbboot-3'">
                <xsl:text>oemboot/suse-SLES11</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="current()" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:attribute>
</xsl:template>

<!-- check SLEPOS boot image -->
<xsl:template match="packages[@type='image']">
    <xsl:copy>
        <!-- copy packages header -->
        <xsl:copy-of select="@*" />
        <!-- parse packages list -->
        <xsl:apply-templates/>
        <!-- add missing SLEPOS packages -->
        <xsl:if test="/image/preferences/type[@boot = '/usr/share/kiwi/image/SLEPOS/netboot-3'
                                           or @boot = '/usr/share/kiwi/image/SLEPOS/isoboot-3'
                                           or @boot = '/usr/share/kiwi/image/SLEPOS/usbboot-3']">
	        <xsl:if test="count(package[@name = 'POS_Image-Netboot-hooks'   ]) = 0">
	            <package name="POS_Image-Netboot-hooks" bootinclude="true" />
	        </xsl:if>
	        <xsl:if test="count(package[@name = 'bootsplash']) = 0">
	            <package name="bootsplash" bootinclude="true" />
	        </xsl:if>
	        <xsl:if test="count(package[@name = 'gfxboot']) = 0">
	            <package name="gfxboot" bootinclude="true" />
	        </xsl:if>
	        <xsl:if test="count(package[@name = 'gfxboot-branding-SLEPOS']) = 0">
	            <package name="gfxboot-branding-SLEPOS" bootinclude="true" />
	        </xsl:if>
	        <xsl:if test="count(package[@name = 'dialog']) = 0">
	            <package name="dialog" bootinclude="true" />
	        </xsl:if>
	        <xsl:if test="count(package[@name = 'bootsplash-branding-SLEPOS']) = 0">
	            <package name="bootsplash-branding-SLEPOS" bootinclude="true" />
	        </xsl:if>
	        <xsl:if test="count(package[@name = 'posbios']) = 0">
	            <package name="posbios" bootinclude="true" />
	        </xsl:if>
        </xsl:if>
    </xsl:copy>
</xsl:template>
</xsl:stylesheet>