#!/usr/bin/perl

use warnings;
use strict;
# GUI part
use yui;
# SLEPOS GUI modules
use POS::Admin::GUI::BS;
use POS::Admin::GUI::IM;
use POS::Admin::GUI::CR;
use POS::Admin::GUI::FIN;
use POS::Admin::GUI::FT;
use POS::Admin::GUI::Data;

use Time::HiRes qw(usleep);

############################################################
# global dialog variables
# widget factory
my $factory = yui::YUI::widgetFactory();
# optional factory
my $optFactory = yui::YUI::optionalWidgetFactory();
# application
my $app = yui::YUI::application();
# dialog object container - contains ID => ( LABEL => '', WIDGET => widget)
my $dialog;
# dialog wizard
my $wizard;
#hint object
my $hint;
# tabs
my $tabSpace;
my $tabBS;
my $tabSV;
my $tabIM;
my $tabCR;
my $tabFIN;
# buttons
my $btnClose;
my $btnBack;
my $btnNext;
my $checkAdvanced;
# tab frame for content
my $tabFrame;

# GUI modules
my $bsconfig;
my $imconfig;
my $crconfig;
my $finconfig;
# Data iface
my $data;
############################################################
# ENV variables - for state tracking and inter-modul comm
my %state = (STATE => 'init', ONADMIN => 0, ADVANCED => 0, FULLCHECK => 0);
my $intState = 'BS';
my $config;

# en lang translation - to be moved into separate file
my %lang = (
    #frame labels
    FRAME_BS  => 'Branch&Server configuration',
    FRAME_IM  => '&Images configuration',
    FRAME_CR  => 'Cash&Register configuration',
    FRAME_FIN => '&Finalize',
    # control boxes
    GLOBAL_ADVANCED     => 'Ad&vanced mode',
    GLOBAL_NEXT         => '&Next',
    GLOBAL_BACK         => '&Back',
    GLOBAL_CANCEL       => '&Quit',
    GLOBAL_BTN_FILENAME => 'S&elect File',
);

sub createDialog
{
    ########################################################################################
    #dialog layout
    $dialog = $factory->createMainDialog();
    # main window
    my $hbox = $factory->createHBox($dialog);
    # hintbox
#    $hint = $factory->createTree($hbox, '');
#    $hint->setWeight(0, 25);
#    $hint->setStretchable(1, 1);
#    $hint->setNotify();
    # right control area
    my $vboxRight = $factory->createVBox($hbox);
    # set to take rest of the space
    $vboxRight->setWeight(0, 75);
    #create Tabspace
    $tabSpace = $optFactory->createDumbTab($vboxRight);
    $tabSpace->setWeight(1, 90);
    $tabSpace->setStretchable(0, 1);
    # create controls
    my $controls = $factory->createHBox($vboxRight);
    $controls->setWeight(1, 10);
    $controls->setStretchable(0, 1);
    $btnClose = $factory->createPushButton($controls, $lang{'GLOBAL_CANCEL'});
    $checkAdvanced = $factory->createCheckBox($controls, $lang{'GLOBAL_ADVANCED'});
    $checkAdvanced->setNotify();
    $factory->createHStretch($controls);
    #createPushButton('GLOBAL_BACK', $controls);
    $btnBack = $factory->createPushButton($controls, $lang{'GLOBAL_BACK'});
    $btnNext = $factory->createPushButton($controls, $lang{'GLOBAL_NEXT'});
    #create Tab header
    $tabBS  = new yui::YItem($lang{'FRAME_BS'},  'BS');
    $tabIM  = new yui::YItem($lang{'FRAME_IM'},  'IM');
    $tabCR  = new yui::YItem($lang{'FRAME_CR'},  'CR');
    $tabFIN = new yui::YItem($lang{'FRAME_FIN'}, 'FIN');
    $tabSpace->addItem($tabBS);
    $tabSpace->addItem($tabIM);
    $tabSpace->addItem($tabCR);
    $tabSpace->addItem($tabFIN);
    $tabSpace->setNotify();
    $tabFrame = $factory->createFrame($tabSpace, '');
    $tabFrame = $factory->createReplacePoint($tabFrame);
    # dialog prepared
    ########################################################################################
} ## end sub createDialog

sub initData
{
    if (defined $ARGV[0] && -f $ARGV[0]) {
        $data->initData($ARGV[0]);
    }
    else {
        $data->initData();
    }
} ## end sub initData

sub updateTabs
{
    $tabFrame->deleteChildren();
    my $state = $state{'STATE'};
    if ($state =~ /BS/) {
        $tabSpace->selectItem($tabBS);
        $bsconfig->drawGUI($tabFrame);
    }
    elsif ($state =~ /IM/) {
        $tabSpace->selectItem($tabIM);
        $imconfig->drawGUI($tabFrame);
    }
    elsif ($state =~ /CR/) {
        $tabSpace->selectItem($tabCR);
        $crconfig->drawGUI($tabFrame);
    }
    elsif ($state =~ /FIN/) {
        $tabSpace->selectItem($tabFIN);
        $finconfig->drawGUI($tabFrame);
    }
    else {
        die 'unsupported state';
    }
    $dialog->recalcLayout();
    $tabFrame->showChild();
} ## end sub updateTabs

sub updateData
{
    my ($newState, $oldState) = @_;
    if (not $newState) {
        $newState = $state{'STATE'};
        $oldState = '';
    }
    if ($newState eq 'BS') {
        if ($oldState and $oldState !~ /BS.*/) {
            $bsconfig->initData($data);
        }
    }
    elsif ($newState eq 'IM') {
        if ($oldState and $oldState !~ /IM.*/) {
            $imconfig->initData($data);
        }
    }
    elsif ($newState eq 'CR') {
        if ($oldState and $oldState !~ /CR.*/) {
            $crconfig->initData($data);
        }
    }
    elsif ($newState eq 'FIN') {
        $finconfig->initData($data);
    }
} ## end sub updateData

sub storeData
{
    if ($state{'STATE'} =~ /BS.*/) {
        return $bsconfig->storeData($data);
    }
    elsif ($state{'STATE'} =~ /IM.*/) {
        return $imconfig->storeData($data);
    }
    elsif ($state{'STATE'} =~ /CR.*/) {
        return $crconfig->storeData($data);
    }
    return 1;
} ## end sub storeData

sub error
{
    my $tmpd = $factory->createPopupDialog();
    my $tmpb = $factory->createVBox($tmpd);
    $factory->createHeading($tmpb, shift);
    $factory->createPushButton($tmpb, '&OK');
    $tmpd->waitForEvent();
    $tmpd->destroy();
} ## end sub error

sub changeState
{
    my $newState = shift;
    if ($newState eq 'BS') {
        $btnBack->setEnabled(0);
        $btnNext->setEnabled(1);
        $checkAdvanced->setEnabled(1);
        $tabSpace->setEnabled(1);
    }
    elsif ($newState eq 'FIN') {
        $btnBack->setEnabled(1);
        $btnNext->setEnabled(0);
        $checkAdvanced->setEnabled(1);
        $tabSpace->setEnabled(1);
        # extra update output structures in case user skip dialogs and to not forget anything
        $state{'FULLCHECK'} = 1;
        # TODO: validate data
        # following is not yet working at it tries to connect to LDAP, which is not desired for now
        # $data->{'OUTDOC'}->validate();
        $state{'FULLCHECK'} = 0;
    }
    elsif ($newState =~ /BSnet|BSservices|BSroles|.*ft|CRdisk/) {
        $btnBack->setEnabled(0);
        $btnNext->setEnabled(0);
        $checkAdvanced->setEnabled(0);
        $tabSpace->setEnabled(0);
    }
    else {
        $btnBack->setEnabled(1);
        $btnNext->setEnabled(1);
        $checkAdvanced->setEnabled(1);
        $tabSpace->setEnabled(1);
    }
    my $oldState = $state{'STATE'};
    $intState = $newState;
    $state{'STATE'} = $newState;
    updateData($state{'STATE'}, $oldState);
    updateTabs();
} ## end sub changeState

#####################################################################################################################
#####################################################################################################################
# MAIN
sub run
{
    # load modules
    # data
    $data = POS::Admin::GUI::Data->new($factory, \%state);
    # gui
    $bsconfig = POS::Admin::GUI::BS->new($factory, $optFactory, \%state);
    $imconfig = POS::Admin::GUI::IM->new($factory, $optFactory, \%state);
    $crconfig = POS::Admin::GUI::CR->new($factory, $optFactory, \%state);
    $finconfig = POS::Admin::GUI::FIN->new($factory, $optFactory, \%state);
    # load XML and defaults
    initData(@ARGV);
    # create basic dialog objects
    createDialog();
    # set first state
    changeState('BS');
    #
    # Event loop
    #
    while (1) {
        my $event = $dialog->pollEvent();
        if (not $event) {
            if ($finconfig->{'IMPORT'} > 0) {
                $finconfig->processEvent();
                next;
            }
            # set normal cursor, fix pollEvent issue
            $app->normalCursor();
            # 15ms wait
            usleep(15000);
            next;
        }
        if ($event->eventType() == $yui::YEvent::CancelEvent) {    # window manager "close window" button
            last;
        }
        # clicking on tabs
        elsif ($event->eventType() == $yui::YEvent::MenuEvent) {
            my $selectedState = $event->item()->iconName();
            next if ($state{'STATE'} eq $selectedState);
            if (storeData()) {
                changeState($selectedState);
            }
            else {
                # reselect original tab
                if ($state{'STATE'} =~ /BS/) {
                    $tabSpace->selectItem($tabBS);
                }
                elsif ($state{'STATE'} =~ /IM/) {
                    $tabSpace->selectItem($tabIM);
                }
                elsif ($state{'STATE'} =~ /CR/) {
                    $tabSpace->selectItem($tabCR);
                }
                elsif ($state{'STATE'} =~ /FIN/) {
                    $tabSpace->selectItem($tabFIN);
                }
                updateTabs();
            }
        }
        my $widget = $event->widget();
        if (not $widget) {
            next;
        }
        if ($widget == $btnClose) {
            last if ($bsconfig->yesnoPopup("\nAre you sure to quit? Unexported changes will be lost.\n"));
        }
        elsif ($widget == $btnNext) {
            unless (storeData()) {
                updateTabs();
                next;
            }
            my $newState;
            if ($state{'STATE'} eq 'BS') {
                $newState = 'IM';
            }
            elsif ($state{'STATE'} eq 'IM') {
                $newState = 'CR';
            }
            elsif ($state{'STATE'} eq 'CR') {
                $newState = 'FIN';
            }
            else {
                next;
            }
            changeState($newState);
        }
        elsif ($widget == $btnBack) {
            unless (storeData()) {
                updateTabs();
                next;
            }
            my $newState;
            if ($state{'STATE'} eq 'IM') {
                $newState = 'BS';
            }
            elsif ($state{'STATE'} eq 'CR') {
                $newState = 'IM';
            }
            elsif ($state{'STATE'} eq 'FIN') {
                $newState = 'CR';
            }
            else {
                next;
            }
            changeState($newState);
        }
        elsif ($widget == $tabSpace) {
            next;
        }
        elsif ($widget == $checkAdvanced) {
            if ($state{'STATE'} =~ /BS.*/) {
                $bsconfig->storeAllWidgetValue();
            }
            elsif ($state{'STATE'} =~ /IM.*/) {
                $imconfig->storeAllWidgetValue();
            }
            elsif ($state{'STATE'} =~ /CR.*/) {
                $crconfig->storeAllWidgetValue();
            }
            elsif ($state{'STATE'} =~ /FIN/) {
                $finconfig->storeAllWidgetValue();
            }
            $state{'ADVANCED'} = $checkAdvanced->value();
            updateData();
            updateTabs();
        }
        ## process event in respective modules
        else {
            my $res;
            if ($state{'STATE'} =~ /BS.*/) {
                $res = $bsconfig->processEvent($event);
            }
            elsif ($state{'STATE'} =~ /IM.*/) {
                $res = $imconfig->processEvent($event);
            }
            elsif ($state{'STATE'} =~ /CR.*/) {
                $res = $crconfig->processEvent($event);
            }
            elsif ($state{'STATE'} =~ /FIN/) {
                $res = $finconfig->processEvent($event);
            }
            if ($res) {
                #force redraw
                updateData();
                updateTabs();
            }
            elsif ($intState ne $state{'STATE'}) {
                #or change state
                changeState($state{'STATE'});
            }
        }
    }
    $data->DESTROY();
    $dialog->destroy();
} ## end sub run

run();
