#!/usr/bin/perl -w

use strict;

use POS::Studio;
use Data::Dumper;
use Getopt::Long;

my $host = undef;
my $user = "admin";
my $pass = undef;
my $template = 'SLES 11 SP3, Just enough OS (JeOS)';
my $force = 0;
my $datadir = '/usr/share/SLEPOS/studio/';

my @tmp     = split('/', $0);
my $appname = pop @tmp;
my $optres  = GetOptions(       # returns true on sucess
    'help|h' => \&help,
    'host=s' => \$host,
    'user=s'  => \$user,
    'pass=s'  => \$pass,
    'template=s' => \$template,
    'force'  => \$force
);  

sub help
{
    print STDERR "Description:\n";
    print STDERR "  Creates slepos images in Suse Studio.\n";
    print STDERR "\n";
    usage();
    exit(0);
} ## end sub help

sub usage
{
    print STDERR "Usage:\n";
    print STDERR "  $appname --host <studio> --user <user> --pass <API Key> [--template <template>] [--force]\n";
    print STDERR "  $appname -h|--help\n";
    print STDERR "\n";
    print STDERR "Options:\n";
    print STDERR "  -h, --help                  displays this help   \n";
    print STDERR "  --host <studio>             Studio Onsite hostname\n";
    print STDERR "  --user <user>               Studio Onsite user\n";
    print STDERR "  --pass <API Key>            Studio Onsite API key\n";
    print STDERR "  --template                  template to base the image on\n";
    print STDERR "                              (default: '$template')\n";
    print STDERR "  --force                     overwrite existing images\n";
    print STDERR "\n";
} ## end sub usage
unless ($optres) {
    print STDERR "Error parsing parameters\n\n";
    usage();
    exit(1);
}

sub add_bootinclude
{
    my ($studio, $app_id) = @_;
    $studio->add_package($app_id, "kexec-tools", 1);
    $studio->add_package($app_id, "POS_Image-Netboot-hooks", 1);
    $studio->add_package($app_id, "bootsplash", 1);
    $studio->add_package($app_id, "gfxboot", 1);
    $studio->add_package($app_id, "gfxboot-branding-SLEPOS", 1);
    $studio->add_package($app_id, "dialog", 1);
    $studio->add_package($app_id, "bootsplash-branding-SLEPOS", 1);
    $studio->add_package($app_id, "posbios", 1);
    $studio->add_package($app_id, "haveged", 1);
}

sub check_image
{
    my ($studio, $app, $force) = @_;
    my $app_id = $studio->get_app_id($app);
    if ($app_id) {
        if ($force) {
            $studio->delete_appliance($app_id);
            check_image($studio, $app, $force);
        }
        else {
            die "'$app' already exists, use --force to overwrite all images"
        }
    }
}

my $studio = POS::Studio->new(
                  'HOST' => $host,
                  'USER' => $user,
                  'PASS' => $pass
                  );

die "Can't login to Studio" unless defined $studio;

my $jeos_id = $studio->get_template_id($template);

die "Can't find the template '$template'" unless $jeos_id;

print STDERR "Checking for existing repos\n";

my $slepos_repo_id_i386 = $studio->get_repo_id('SLEPOS 11 SP3 i386');

die "Can't find the 'SLEPOS 11 SP3 i386' repository" unless $slepos_repo_id_i386;

my $slepos_up_repo_id_i386 = $studio->get_repo_id('SLEPOS 11 SP3 Updates i386');

die "Can't find the 'SLEPOS 11 SP3 Updates i386' repository" unless $slepos_up_repo_id_i386;

my $slepos_repo_id_x86_64 = $studio->get_repo_id('SLEPOS 11 SP3 x86_64');

die "Can't find the 'SLEPOS 11 SP3 x86_64' repository" unless $slepos_repo_id_x86_64;

my $slepos_up_repo_id_x86_64 = $studio->get_repo_id('SLEPOS 11 SP3 Updates x86_64');

die "Can't find the 'SLEPOS 11 SP3 Updates x86_64' repository" unless $slepos_up_repo_id_x86_64;


#my $sle11_sp1_sdk_i386 = $studio->get_repo_id('SLE 11 SP1 SDK i386');
#my $sle11_sp1_sdk_updates_i386 = $studio->get_repo_id('SLE 11 SP1 SDK Updates i386');
#my $sle11_sp2_sdk_i386 = $studio->get_repo_id('SLE 11 SP2 SDK i386');
#my $sle11_sp2_sdk_updates_i386 = $studio->get_repo_id('SLE 11 SP2 SDK Updates i386');
my $sle11_sp3_sdk_i386 = $studio->get_repo_id('SLE 11 SP3 SDK i386');
my $sle11_sp3_sdk_updates_i386 = $studio->get_repo_id('SLE 11 SP3 SDK Updates i386');

#my $sle11_sp1_sdk_x86_64 = $studio->get_repo_id('SLE 11 SP1 SDK x86_64');
#my $sle11_sp1_sdk_updates_x86_64 = $studio->get_repo_id('SLE 11 SP1 SDK Updates x86_64');
#my $sle11_sp2_sdk_x86_64 = $studio->get_repo_id('SLE 11 SP2 SDK x86_64');
#my $sle11_sp2_sdk_updates_x86_64 = $studio->get_repo_id('SLE 11 SP2 SDK Updates x86_64');
my $sle11_sp3_sdk_x86_64 = $studio->get_repo_id('SLE 11 SP3 SDK x86_64');
my $sle11_sp3_sdk_updates_x86_64 = $studio->get_repo_id('SLE 11 SP3 SDK Updates x86_64');

print STDERR "Checking for existing images\n";

check_image($studio, "POS_Image_JeOS4", $force);
check_image($studio, "POS_Image_Graphical4", $force);
check_image($studio, "POS_Image_Adminserver4", $force);
check_image($studio, "POS_Image_Branchserver4", $force);
check_image($studio, "POS_Image_Comboserver4", $force);

#JeOS image
print STDERR "Creating JeOS image\n";

my $jeos_app_id = $studio->clone_appliance($jeos_id, "POS_Image_JeOS4", "i686");
$studio->add_repo($jeos_app_id, $slepos_repo_id_i386);
$studio->add_repo($jeos_app_id, $slepos_up_repo_id_i386);
#$studio->remove_repo($jeos_app_id, $sle11_sp1_sdk_i386) if $sle11_sp1_sdk_i386; 
#$studio->remove_repo($jeos_app_id, $sle11_sp1_sdk_updates_i386) if $sle11_sp1_sdk_updates_i386;
#$studio->remove_repo($jeos_app_id, $sle11_sp2_sdk_i386) if $sle11_sp2_sdk_i386;
#$studio->remove_repo($jeos_app_id, $sle11_sp2_sdk_updates_i386) if $sle11_sp2_sdk_updates_i386;
$studio->remove_repo($jeos_app_id, $sle11_sp3_sdk_i386) if $sle11_sp3_sdk_i386;
$studio->remove_repo($jeos_app_id, $sle11_sp3_sdk_updates_i386) if $sle11_sp3_sdk_updates_i386;

$studio->add_pattern($jeos_app_id, "SLEPOS_Template_Minimal");

add_bootinclude($studio, $jeos_app_id);
$studio->add_build_script_local($jeos_app_id, $datadir . 'build_script.sh');
$studio->add_configuration($jeos_app_id, "
<configuration>
  <version>4.0.0</version>
  <type>net</type>
  <settings>
    <runlevel>3</runlevel>
  </settings>
</configuration>
");
$studio->add_background($jeos_app_id, $datadir . 'back.jpg');
my $filename = $studio->export_kiwi_configuration($jeos_app_id, "net");
if ($filename) {
    print STDERR "Saved $filename\n";
} else {
    print STDERR "Image can't be saved\n";
}

#Graphical image
print STDERR "Creating Graphical image\n";

my $graphical_app_id = $studio->clone_appliance($jeos_id, "POS_Image_Graphical4", "i686");
$studio->add_repo($graphical_app_id, $slepos_repo_id_i386);
$studio->add_repo($graphical_app_id, $slepos_up_repo_id_i386);
#$studio->remove_repo($graphical_app_id, $sle11_sp1_sdk_i386) if $sle11_sp1_sdk_i386;
#$studio->remove_repo($graphical_app_id, $sle11_sp1_sdk_updates_i386) if $sle11_sp1_sdk_updates_i386;
#$studio->remove_repo($graphical_app_id, $sle11_sp2_sdk_i386) if $sle11_sp2_sdk_i386;
#$studio->remove_repo($graphical_app_id, $sle11_sp2_sdk_updates_i386) if $sle11_sp2_sdk_updates_i386;
$studio->remove_repo($graphical_app_id, $sle11_sp3_sdk_i386) if $sle11_sp3_sdk_i386;
$studio->remove_repo($graphical_app_id, $sle11_sp3_sdk_updates_i386) if $sle11_sp3_sdk_updates_i386;
$studio->add_pattern($graphical_app_id, "SLEPOS_Template_Graphical");
$studio->add_package($graphical_app_id, "java-1_7_0-ibm");
add_bootinclude($studio, $graphical_app_id);
$studio->add_build_script_local($graphical_app_id, $datadir . 'build_script.sh');
$studio->add_configuration($graphical_app_id, "
<configuration>
  <version>4.0.0</version>
  <type>net</type>
  <settings>
    <runlevel>5</runlevel>
  </settings>
</configuration>
");
$studio->add_background($graphical_app_id, $datadir . 'back.jpg');
$filename = $studio->export_kiwi_configuration($graphical_app_id, "net");
if ($filename) {
    print STDERR "Saved $filename\n";
} else {
    print STDERR "Image can't be saved\n";
}

#Branchserver image
print STDERR "Creating Branchserver image\n";

my $branchserver_app_id = $studio->clone_appliance($jeos_id, "POS_Image_Branchserver4", "x86_64");
$studio->add_repo($branchserver_app_id, $slepos_repo_id_x86_64);
$studio->add_repo($branchserver_app_id, $slepos_up_repo_id_x86_64);
#$studio->remove_repo($branchserver_app_id, $sle11_sp1_sdk_x86_64) if $sle11_sp1_sdk_x86_64;
#$studio->remove_repo($branchserver_app_id, $sle11_sp1_sdk_updates_x86_64) if $sle11_sp1_sdk_updates_x86_64;
#$studio->remove_repo($branchserver_app_id, $sle11_sp2_sdk_x86_64) if $sle11_sp2_sdk_x86_64;
#$studio->remove_repo($branchserver_app_id, $sle11_sp2_sdk_updates_x86_64) if $sle11_sp2_sdk_updates_x86_64;
$studio->remove_repo($branchserver_app_id, $sle11_sp3_sdk_x86_64) if $sle11_sp3_sdk_x86_64;
$studio->remove_repo($branchserver_app_id, $sle11_sp3_sdk_updates_x86_64) if $sle11_sp3_sdk_updates_x86_64;

$studio->add_pattern($branchserver_app_id, "SLEPOS_Template_Graphical");
$studio->add_pattern($branchserver_app_id, "SLEPOS_Server_Branch");
$studio->add_pattern($branchserver_app_id, "x11");

$studio->add_package($branchserver_app_id, "yast2-firstboot");
$studio->add_package($branchserver_app_id, "yast2-pos-installation");
$studio->add_package($branchserver_app_id, "yast2-users");

$studio->add_overlay_file($branchserver_app_id, 'reconfig_system', '/var/lib/YaST2', 'root', 'root', '644', '');
$studio->add_overlay_file_local($branchserver_app_id, 'firstboot.xml', '/etc/YaST2', 'root', 'root', '644', $datadir . 'config-yast-firstboot.xml');

$studio->add_build_script_local($branchserver_app_id, $datadir . 'build_script.sh');
$studio->add_configuration($branchserver_app_id, "
<configuration>
  <version>4.0.0</version>
  <type>oemstick</type>
  <settings>
    <runlevel>5</runlevel>
  </settings>
</configuration>
");
$studio->add_background($branchserver_app_id, $datadir . 'back.jpg');
$filename = $studio->export_kiwi_configuration($branchserver_app_id, "oemstick");
if ($filename) {
    print STDERR "Saved $filename\n";
} else {
    print STDERR "Image can't be saved\n";
}

#Adminserver image
print STDERR "Creating Adminserver image\n";

my $adminserver_app_id = $studio->clone_appliance($jeos_id, "POS_Image_Adminserver4", "x86_64");
$studio->add_repo($adminserver_app_id, $slepos_repo_id_x86_64);
$studio->add_repo($adminserver_app_id, $slepos_up_repo_id_x86_64);
#$studio->remove_repo($adminserver_app_id, $sle11_sp1_sdk_x86_64) if $sle11_sp1_sdk_x86_64;
#$studio->remove_repo($adminserver_app_id, $sle11_sp1_sdk_updates_x86_64) if $sle11_sp1_sdk_updates_x86_64;
#$studio->remove_repo($adminserver_app_id, $sle11_sp2_sdk_x86_64) if $sle11_sp2_sdk_x86_64;
#$studio->remove_repo($adminserver_app_id, $sle11_sp2_sdk_updates_x86_64) if $sle11_sp2_sdk_updates_x86_64;
$studio->remove_repo($adminserver_app_id, $sle11_sp3_sdk_x86_64) if $sle11_sp3_sdk_x86_64;
$studio->remove_repo($adminserver_app_id, $sle11_sp3_sdk_updates_x86_64) if $sle11_sp3_sdk_updates_x86_64;

$studio->add_pattern($adminserver_app_id, "SLEPOS_Template_Graphical");
$studio->add_pattern($adminserver_app_id, "SLEPOS_Server_Admin");
$studio->add_pattern($adminserver_app_id, "SLEPOS_Server_Image");
$studio->add_pattern($adminserver_app_id, "x11");
$studio->add_build_script_local($adminserver_app_id, $datadir . 'build_script.sh');
$studio->add_configuration($adminserver_app_id, "
<configuration>
  <version>4.0.0</version>
  <type>oemstick</type>
  <settings>
    <runlevel>5</runlevel>
  </settings>
</configuration>
");
$studio->add_background($adminserver_app_id, $datadir . 'back.jpg');
$filename = $studio->export_kiwi_configuration($adminserver_app_id, "oemstick");
if ($filename) {
    print STDERR "Saved $filename\n";
} else {
    print STDERR "Image can't be saved\n";
}


#Combo server image
print STDERR "Creating Comboserver image\n";

my $comboserver_app_id = $studio->clone_appliance($jeos_id, "POS_Image_Comboserver4", "x86_64");
$studio->add_repo($comboserver_app_id, $slepos_repo_id_x86_64);
$studio->add_repo($comboserver_app_id, $slepos_up_repo_id_x86_64);
#$studio->remove_repo($comboserver_app_id, $sle11_sp1_sdk_x86_64) if $sle11_sp1_sdk_x86_64;
#$studio->remove_repo($comboserver_app_id, $sle11_sp1_sdk_updates_x86_64) if $sle11_sp1_sdk_updates_x86_64;
#$studio->remove_repo($comboserver_app_id, $sle11_sp2_sdk_x86_64) if $sle11_sp2_sdk_x86_64;
#$studio->remove_repo($comboserver_app_id, $sle11_sp2_sdk_updates_x86_64) if $sle11_sp2_sdk_updates_x86_64;
$studio->remove_repo($comboserver_app_id, $sle11_sp3_sdk_x86_64) if $sle11_sp3_sdk_x86_64;
$studio->remove_repo($comboserver_app_id, $sle11_sp3_sdk_updates_x86_64) if $sle11_sp3_sdk_updates_x86_64;

$studio->add_pattern($comboserver_app_id, "SLEPOS_Template_Graphical");
$studio->add_pattern($comboserver_app_id, "SLEPOS_Server_Admin");
$studio->add_pattern($comboserver_app_id, "SLEPOS_Server_Image");
$studio->add_pattern($comboserver_app_id, "SLEPOS_Server_Branch");
$studio->add_pattern($comboserver_app_id, "x11");
$studio->add_build_script_local($comboserver_app_id, $datadir . 'build_script.sh');
$studio->add_configuration($comboserver_app_id, "
<configuration>
  <version>4.0.0</version>
  <type>oemstick</type>
  <settings>
    <runlevel>5</runlevel>
  </settings>
</configuration>
");
$studio->add_background($comboserver_app_id, $datadir . 'back.jpg');
$filename = $studio->export_kiwi_configuration($comboserver_app_id, "oemstick");
if ($filename) {
    print STDERR "Saved $filename\n";
} else {
    print STDERR "Image can't be saved\n";
}

