#!/usr/bin/perl -w

# $Id: posldap2dhcp.pl,v 1.3 2003/09/10 11:27:34 frankb Exp $

#
# Copyright (c) 2001 SuSE Linux Solutions AG, Germany.  All rights reserved.
#
# Author: Stephan Dhr, 20010412
# Author: Frank Balzer, 2003
# ldap2dhcpd.pl:
#       read entries from the LDAP server,
#       generates a new dhcpd.conf file
#       and reload the daemon (if neccessary)

use Getopt::Long;
use Net::IP;
use Net::IPv4Addr;
use POS::LDAP;
use File::Compare;
use File::Copy;
use IO::File qw(:DEFAULT !SEEK_SET !SEEK_END !SEEK_CUR);    #to resolve namespace conflict with POSIX
use POSIX;
use POS::Config;
use POS::Syslog;
use POS::Pid;
use POS::LdapUtil;
use POS::posDhcp;
use POS::posDataStatic;
use POS::posDataCache;

use strict;

log_parse_argv();
##Globals and fallback definitions

my %POS_GLOBALS;

#init static and dynamic cache here
my $static_obj = POS::posDataStatic->new();
if (!defined $static_obj) {    #error already processed in the module
    exit(1);
}

my $cache_obj = POS::posDataCache->new('STATIC' => $static_obj);
if (!defined $cache_obj) {     #error already processed in the module
    exit(1);
}

$POS_GLOBALS{'STATIC'} = $static_obj;
$POS_GLOBALS{'CACHE'}  = $cache_obj;

# parse command-line options
# config file options can be overwritten by command line options
if (!GetOptions(
        'templatefile=s'   => \$static_obj->{'CONFIG'}{'LDAP2DHCP_TEMPLATEFILE'},
        'dhcpconffile=s'   => \$static_obj->{'CONFIG'}{'LDAP2DHCP_DHCPCONFFILE'},
        'notauthoritative' => \$static_obj->{'CONFIG'}{'LDAP2DHCP_NOTAUTHORITATIVE'},
        'nodynrange'       => \$static_obj->{'CONFIG'}{'LDAP2DHCP_NODYNRANGE'},
    ) ||
    $#ARGV >= 0
  ) {
    log_alert_print('called with invalid command-line parameters');
    print_usage();
    exit 1;
}


# check if already running #####
$SIG{'INT'} = 'break';
my $pid_obj = POS::Pid->new(TIMEOUT => $POS_GLOBALS{'PIDTIMEOUT'}, DIRECTORY => $POS_GLOBALS{'PIDDIR'});

if ($pid_obj->{'ERROR'}) {
    log_alert_print($pid_obj->{'ERROR_MSG'});
    exit(1);
}

my $dhcp = POS::posDhcp->new(%POS_GLOBALS);

exit $dhcp->ldap2dhcp();

#############################
# end of main program
#############################


sub print_usage
{
    print STDERR "usage:\n";
    #    print STDERR "  --ldaphost=<ldap hostname>\n";
    #    print STDERR "  --ldapbase=<ldap base>\n";
    print STDERR "  --templatefile=<template file for dhcpd.conf>\n";
    print STDERR "  --dhcpconffile=<name of dhcpd.conf file to be written>\n";
    print STDERR "optional parameters:\n";
    print STDERR "  --nodynrange (no dynamic ranges)\n";
    print STDERR "  --notauthoritative (don't send DHCPNAKs)\n";
    log_print_usage();


    print STDERR "\n";
    print STDERR "Lease times must be specified in LDAP\n";
    print STDERR "(under scService named 'dhcp')" . " like this:\n";
    print STDERR "## lease time dyn. range (1 <= n < 600)\n";
    print STDERR "scDhcpDynLeaseTime=60\n";
    print STDERR "## lease time fixed range (>= 3600)\n";
    print STDERR "scDhcpFixedLeaseTime=14400\n";
} ## end sub print_usage


END {
#delete only if pid assigned (no early ctrl+c), not forked to other or if pidfile is ours and not of some other instance already runing
    $pid_obj->delete() if defined $pid_obj && $$ == $pid_obj->get_pid() && $$ == $pid_obj->get_pidf();
} ## end END

sub break
{
    #main reason is to invoke END block in the ctrl+C evend to clear the pid file
    printf("\nctrl+C detected! \n");
    exit(1);
} ## end sub break

