#!/usr/bin/perl -w
## @file
# Downloads and sychronizes boot and system images from the adminserver to the branchserver.
#
# @par Usage
#  - possyncimages.pl
#  - possyncimages.pl -h|--help
#
#  @param -h,--help    displays help infomation
#
# TODO: add further parameters, e.g. for different mode of download for delta image functionality

use strict;

use POS::LDAP;
use POS::Config;
use POS::posDataStatic;
use POS::posDataCache;
use POS::Pid;
use POS::Syslog;
use POS::SyncImages;
use Getopt::Long;

my %POS_GLOBALS;
$POS_GLOBALS{'REMOTE'} = 0;
$POS_GLOBALS{'LOCAL'}  = 0;

log_parse_argv();
my @tmp     = split('/', $0);
my $appname = pop @tmp;
my $optres  = GetOptions(       # returns true on sucess
    'help|h' => \&help,
    'remote' => \$POS_GLOBALS{'REMOTE'},
    'local'  => \$POS_GLOBALS{'LOCAL'}
);    # TODO: add further parameters, e.g. for different mode of download for delta image functionality

sub help
{
    print STDERR "Description:\n";
    print STDERR "  Downloads and sychronizes boot and system images from the adminserver to the branchserver.\n";
    print STDERR "\n";
    usage();
    exit(0);
} ## end sub help

sub usage
{
    print STDERR "Usage:\n";
    print STDERR "  $appname [--local] [--remote]\n";
    print STDERR "  $appname -h|--help\n";
    print STDERR "\n";
    print STDERR "Options:\n";
    print STDERR "  -h, --help                  displays this help   \n";
    print STDERR "  --remote                    download images from the remote Admin Server\n";
    print STDERR "  --local                     locally install the downloaded images   \n";
    log_print_usage();
    print STDERR "\n";
} ## end sub usage
unless ($optres) {
    print STDERR "Error parsing parameters\n\n";
    usage();
    exit(1);
}


$SIG{'INT'} = 'break';
my $pid = POS::Pid->new(TIMEOUT => 5);
if ($pid->{'ERROR'}) {
    log_alert_printf("already running: $pid->{'ERROR_MSG'}");
    exit(1);
}

if (!$POS_GLOBALS{'REMOTE'} && !$POS_GLOBALS{'LOCAL'}) {
    #no options specified, do all
    $POS_GLOBALS{'REMOTE'} = 1;
    $POS_GLOBALS{'LOCAL'}  = 1;
}

#init static and dynamic cache here
my $static_obj = POS::posDataStatic->new();
if (!defined $static_obj) {    #error already processed in the module
    exit(1);
}

my $cache_obj = POS::posDataCache->new('STATIC' => $static_obj);
if (!defined $cache_obj) {     #error already processed in the module
    exit(1);
}

if ($POS_GLOBALS{'REMOTE'} &&
    defined $static_obj->{'CONFIG'}{'POS_ADMIN_DEPLOYED'} &&
    $static_obj->{'CONFIG'}{'POS_ADMIN_DEPLOYED'} eq "yes") {
    log_warn_print("Running on admin server, disabling --remote option");
    $POS_GLOBALS{'REMOTE'} = 0;
}

$POS_GLOBALS{'STATIC'} = $static_obj;
$POS_GLOBALS{'CACHE'}  = $cache_obj;

my $sync = POS::SyncImages->new(%POS_GLOBALS);

exit($sync->sync() ? 0 : 1);

END {
#delete only if pid assigned (no early ctrl+c), not forked to other or if pidfile is ours and not of some other instance already runing
    $pid->delete() if defined $pid && $$ == $pid->get_pid() && $$ == $pid->get_pidf();
} ## end END

sub break
{
    #main reason is to invoke END block in the ctrl+C evend to clear the pid file
    log_error_printf("\nctrl+C detected!");
    exit(1);
} ## end sub break

