#!/bin/bash
set -e

function importFile {
	# /.../
	# import the config.<MAC> style format. the function
	# will export each entry of the file as variable into
	# the current shell environment
	# ----
	IFS_ORIG=$IFS
	IFS="
	"
	while read line;do
		echo $line | grep -qi "^#" && continue
		key=`echo "$line" | cut -d '=' -f1`
		item=`echo "$line" | cut -d '=' -f2-`
		if [ -z "$key" ] || [ -z "$item" ];then
			continue
		fi
		if ! echo $item | grep -E -q "^(\"|')";then
			item="'"$item"'"
		fi
		eval export "$key\=$item"
	done
	IFS=$IFS_ORIG
}


TFTP_HOST=tftp
CHANGE_ROLE=
CHANGE_ID=

TMPDIR=`mktemp -d`

usage () {
    cat << EOF_USAGE
    
    usage:
    $0 --mac <MAC> [--id <id>] [--role <role_dn>]
    
EOF_USAGE
}
    
TEMP=`getopt -o is -l mac:,id:,role: -n "test_role_api.sh" -- "$@"`
# Note the quotes around `$TEMP': they are essential!. 

if [ $? != 0 ] ; then usage 1>&2; exit 1; fi

eval set -- "$TEMP"

while true; do
    case $1 in
        --mac)
            MAC=$2
            shift 2
            ;;
        --role)
            SELECTED_ROLE=$2
            CHANGE_ROLE=1
            shift 2
            ;;
        --id)
            SELECTED_ID=$2
            CHANGE_ID=1
            shift 2
            ;;

        --) shift ; break ;;
        
        *)
            echo "Internal error!"
            exit 254
            ;;
    esac
done

if [ $# -gt 0 ]; then
    echo "Odd addition arguments '$*'\n".
fi

if [ -z "$MAC" ]; then
    echo "missing --mac argument"
    exit 1
fi

# get the current configuration
if atftp -g -l "$TMPDIR/config" -r "KIWI/config.$MAC" $TFTP_HOST ; then
    importFile < "$TMPDIR/config"
else
    echo "$TFTP_KIWI/config.$MAC not found"
    echo "The workstation is probably not registered yet"
    exit 1
fi

echo >"$TMPDIR/hwtype"
# role/id should be included only if we want change them
# note that there is a difference between POS_ID= (which clears the ID) 
# and missing POS_ID (which keeps it unchanged)
#
if [ -n "$CHANGE_ID" ]; then
    echo "POS_ID=$SELECTED_ID"      >>"$TMPDIR/hwtype"
fi
if [ -n "$CHANGE_ROLE" ]; then
    echo "POS_ROLE=$SELECTED_ROLE"  >>"$TMPDIR/hwtype"
fi

# upload the file with .HASH suffix, HASH is MD5 of the file content
HASH=`md5sum $TMPDIR/hwtype |cut -d ' ' -f 1`
if ! atftp -p -l "$TMPDIR/hwtype" -r "upload/hwtype.$MAC.$HASH" $TFTP_HOST ; then
    echo "Upload of upload/hwtype.$MAC.$HASH failed"
    exit 1
fi

# wait until the uploaded file is processed
while atftp -g -l "$TMPDIR/hwtype_test" -r "upload/hwtype.$MAC.$HASH" $TFTP_HOST 2>/dev/null ; do
    echo "Waiting until the upload/hwtype.$MAC.$HASH file is processed"
    sleep 1
done

# wait until the new config appears
while ! atftp -g -l "$TMPDIR/config" -r "KIWI/config.$MAC" $TFTP_HOST 2>/dev/null; do
    echo "Waiting for KIWI/config.$MAC"
    sleep 1
done

# parse the new config
unset POS_ID
unset POS_ROLE
unset IMAGE
unset POS_HWTYPE_ERR_HASH
unset POS_ERR
importFile < "$TMPDIR/config"

#cleanup
rm -f "$TMPDIR/config" "$TMPDIR/hwtype" "$TMPDIR/hwtype_test"
rmdir "$TMPDIR"

#process the reply
if [ -z "$CHANGE_ID" ]; then
    SELECTED_ID=$POS_ID
fi

if [ -z "$CHANGE_ROLE" ]; then
    SELECTED_ROLE=$POS_ROLE
fi

if [ "$HASH" = "$POS_HWTYPE_ERR_HASH" ]; then
    #same hash - the error message is for us
    echo "Server responded with an error:"
    echo "$POS_ERR"
    exit 1
elif [ -n "$IMAGE" -a "$SELECTED_ID" = "$POS_ID" -a "$SELECTED_ROLE" = "$POS_ROLE" ]; then
    # IMAGE is set and we got the id/role that we wanted - everything is OK
    echo "ID/Role was changed successfully to '$POS_ID'/'$POS_ROLE'"
    exit 0
else
    # we got something we didn't want
    # we can try again
    echo "There was probably a conflict with another client"
    echo "The workstation is configured for '$POS_ID'/'$POS_ROLE', IMAGE='$IMAGE'"
    echo "You may try to run this command again"
    exit 1
fi


